create or replace package un$schema_check authid current_user as
--------------------------------------------------------------------------------
-- 'Last revision: 27.05.2009'
--------------------------------------------------------------------------------
procedure set_output(p_set varchar2); -- Set output mode: O, T
-- including 'O' means using dbms_output (default)
-- including 'T' means using table xschemastate
procedure clear_output_table; -- truncate table xschemastate reuse storage
--------------------------------------------------------------------------------
procedure check_lost_chars; -- Test PL/SQL objects for '????' pattern
procedure check_cm_sptype; -- Test TMDB_CM for lost and invalid pairs
procedure check_txsld; -- Test saldo from TMDB_CM and TXSLD
procedure check_sequences; -- Test ID_TMDB_DOCS and ID_TMS_UNIVERS
procedure check_tuning_issues; --    /   
--------------------------------------------------------------------------------
procedure run_all;
--------------------------------------------------------------------------------
end;
/
create or replace package body un$schema_check as
--------------------------------------------------------------------------------
-- 'Last revision: 27.05.2009'
--------------------------------------------------------------------------------
procedure set_output(p_set varchar2)is
begin
 envun4.EnvSetValue('un$schema_check.output_mode',p_set);
end;
--------------------------------------------------------------------------------
procedure set_output_time is
begin
 envun4.EnvSetValue('un$schema_check.output_time',
   to_char(sysdate,'dd.mm.yyyy hh24:mi:ss'));
end;
--------------------------------------------------------------------------------
function output_time return varchar2 is
 v_time varchar2(20):=sys_context('envun4','un$schema_check.output_time',20);
begin
 if v_time is null then
  v_time:=to_char(sysdate,'dd.mm.yyyy hh24:mi:ss');
 end if;
 return v_time;
end;
--------------------------------------------------------------------------------
function use_output_std return boolean is
begin
 if instr(nvl(sys_context('envun4','un$schema_check.output_mode'),'O'),'O')>0 then
  return true;
 end if;
 return false;
end;
--------------------------------------------------------------------------------
function use_output_table return boolean is
begin
 if instr(sys_context('envun4','un$schema_check.output_mode'),'T')>0 then
  return true;
 end if;
 return false;
end;
--------------------------------------------------------------------------------
procedure put(p_name varchar2,p_type varchar2,p_code varchar2,p_text varchar2)is
begin
 if use_output_std then
  say(output_time||': '||p_type||' '||p_name||' - '||p_code||chr(10)||p_text,255);
 end if;
 --
 if use_output_table then
  execute immediate 'insert into xschemastate(id_time,object_name,object_type'||
    ',error_code,error_text) values (:1,:2,:3,:4,:5)'
  using to_date(output_time,'dd.mm.yyyy hh24:mi:ss'),p_name,p_type,p_code,p_text;
 end if;
end;
--------------------------------------------------------------------------------
procedure check_output_table(p_error varchar2:=null) is
begin
 if not use_output_table then
  return;
 end if;
 --
 for c in (select null from user_tables where table_name=upper('xschemastate')) loop
  if p_error is null then
   execute immediate 'truncate table xschemastate reuse storage';
  else
   execute immediate 'delete from xschemastate where error_code=:1' using p_error;
  end if;
  return;
 end loop;
 --
 execute immediate 'create table xschemastate (id_time date'||
   ',object_name varchar2(30),object_type varchar2(30)'||
   ',error_code varchar2(30),error_text varchar2(4000))';
end;
--------------------------------------------------------------------------------
procedure clear_output_table is
 e_table_not_exists exception;pragma exception_init(e_table_not_exists,-942);
begin
 execute immediate 'truncate table xschemastate reuse storage';
exception when e_table_not_exists then
 check_output_table;
end;
--------------------------------------------------------------------------------
procedure check_lost_chars is -- Test PL/SQL objects for '????' pattern
 c_error constant varchar2(20):='Lost chars';
begin
 check_output_table(c_error);
 set_output_time;
 --
 for c in (select name,type,line,rtrim(text,chr(10))text from user_source a
   where text like '%????%' and line= (select min(line) from user_source
     where name=a.name and type=a.type and text like '%????%')) loop
  put(c.name,c.type,c_error,'Line '||c.line||': "'||c.text||'"');
 end loop;
 --
 commit;
end;
--------------------------------------------------------------------------------
procedure check_cm_sptype is -- Test TMDB_CM for lost and invalid pairs
 v_sql long;
 --v_count int;
 v_cursor sys_refcursor;
 v_nrdoc int;
 c_error constant varchar2(20):='Invalid CM pairs';
begin
 check_output_table(c_error);
 set_output_time;
 --
 --v_sql:='select count(distinct nrdoc)
 v_sql:='select distinct nrdoc
from(
select data,nrdoc,cod,sptype,cont,cont1,sc,dep,sc1,annrdoc,strsc,nrcm,andata,valuta
,sum(suma)suma,sum(cant)cant,sum(sumaval)sumaval,sum(sumagaap)sumagaap,sum(cant1)cant1
from(
select data,nrdoc,cod,sptype,ct cont,ct1 cont1,ctsc sc,ctdep dep,ctsc1 sc1
,ctnrdoc annrdoc,ctstrsc strsc,ctnrcm nrcm,ctdata andata,valutact valuta
,sum(suma)suma,sum(cant)cant,sum(sumavalct)sumaval,sum(sumagaap)sumagaap,sum(ctcant1)cant1
from tmdb_cm
where sptype in(0,2) and ct is not null
group by data,nrdoc,cod,sptype,ct,ct1,ctsc,ctdep,ctsc1,ctnrdoc,ctstrsc,ctnrcm,ctdata,valutact
union all
select data,nrdoc,cod,sptype-1,dt cont,dt1 cont1,dtsc sc,dtdep dep,dtsc1 sc1
,dtnrdoc annrdoc,dtstrsc strsc,dtnrcm nrcm,dtdata andata,valutadt valuta
,sum(suma)suma,sum(cant)cant,sum(sumavaldt)sumaval,sum(sumagaap)sumagaap,sum(dtcant1)cant1
from tmdb_cm
where sptype in(1,3) and dt is not null
group by data,nrdoc,cod,sptype,dt,dt1,dtsc,dtdep,dtsc1,dtnrdoc,dtstrsc,dtnrcm,dtdata,valutadt
)group by data,nrdoc,cod,sptype,cont,cont1,sc,dep,sc1,annrdoc,strsc,nrcm,andata,valuta
having sum(suma)<>0 or sum(cant)<>0 or sum(sumaval)<>0 or sum(sumagaap)<>0 or sum(cant1)<>0
)';
-- execute immediate v_sql into v_count;
-- --
-- if v_count>0 then
--  put('TMDB_CM','TABLE','check_cm_sptype','There are '||v_count||
--    ' documents containing invalid pairs in CM');
-- end if;
 open v_cursor for v_sql;
 loop
  fetch v_cursor into v_nrdoc;
  exit when v_cursor%notfound;
  put('TMDB_CM','TABLE','c_error'
    ,'Document '||v_nrdoc||' contains invalid pairs in CM');
 end loop;
 close v_cursor;
 --
 commit;
end;
--------------------------------------------------------------------------------
-- Test saldo from TMDB_CM and TXSLD
procedure check_txsld is
 v_count int;
 e_table_not_exists exception;pragma exception_init(e_table_not_exists,-942);
 c_error constant varchar2(20):='Actual balance';
begin
 begin
  execute immediate 'select count(*) from txsld' into v_count;
 exception when e_table_not_exists then
  return; -- Table does not exist
 end;
 --
 check_output_table(c_error);
 --
 if v_count=0 then -- Table is empty
  for c in (select null from user_triggers -- Trigger enabled
    where trigger_name='TMDB_CM_TSLD_TR' and status='ENABLED') loop
   set_output_time;
   put('TXSLD','TABLE','No data in TXSLD',
     '     .'||chr(10)||
     ': call un$sld.rebuild(''TXSLD'')');
  end loop;
  return; -- Actual saldo is not used
 end if;
 --
 un$sld.make(pTestMode=>true);
 --
 execute immediate 'select count(*) from vsld' into v_count;
 if v_count>0 then
  set_output_time;
  put('TXSLD','TABLE','Invalid data in TXSLD',
    '     .'||chr(10)||
    ': call un$sld.rebuild(''TXSLD'')');
 end if;
 --
 commit;
end;
--------------------------------------------------------------------------------
-- Test ID_TMDB_DOCS and ID_TMS_UNIVERS
procedure check_sequences is
 procedure check_one(p_seq varchar2,p_tbl varchar2,p_fld varchar2)is
  v_last int;
  v_real int;
 begin
  select last_number into v_last
  from user_sequences where sequence_name=p_seq;
  --
  execute immediate 'select nvl(max('||p_fld||'),0) from '||p_tbl into v_real;
  --
  if v_real<v_last then
   return;
  end if;
  --
  put(p_seq,'SEQUENCE','Sequence value is too small',
    ' '||p_seq||'   '||v_last||','||chr(10)||
    '   '||p_tbl||'   '||p_fld||'  '||v_real||chr(10)||
    ': call sys_setsquenc('''||p_seq||''','||v_real||')');
 end;
 --
begin
 check_one('ID_TMDB_DOCS','TMDB_DOCS','COD');
 check_one('ID_TMS_UNIVERS','TMS_UNIVERS','COD');
end;
--------------------------------------------------------------------------------
--    /   
procedure check_tuning_issues is
 e_table_not_exists exception;pragma exception_init(e_table_not_exists,-942);
 n number;
 total_MB number;
 lost_MB number;
 lost_prc number;
 pct_parsing number;
 v_sql long;
 c_error constant varchar2(20):='Tuning issues';
begin
 check_output_table(c_error);
 set_output_time;

 --     XCLOB
 begin
  select num_rows into n from all_tables
  where owner='UN4PUBLIC' and table_name = 'XCLOB';
  --
  if n is null then
   set_output_time;
   put('XCLOB','Missing statistics',c_error,
     '   XCLOB   .'||chr(10)||
     '      UN4PUBLIC  SYSDBA:'||chr(10)||
     ''||chr(10)||
     'begin'||chr(10)||
     'dbms_stats.set_table_stats(ownname=>''UN4PUBLIC'',tabname=>''XCLOB'''||chr(10)||
     ',numrows => 1,numblks=>2,avgrlen=>4096,cachedblk=>2,cachehit=>0.95);'||chr(10)||
     'end;'||chr(10)||
     ''||chr(10)||
     '    dynamic sampling     .');
  end if;
 exception when no_data_found then
   put('XCLOB','Missing statistics',c_error,
     ' UN4PUBLIC.XCLOB     ');
 end;

 --     'KGH: NO ACCESS' (%    SGA)
 v_sql :=
 'select total_MB, lost_MB, round(lost_MB*100/total_MB,2) lost_prc from
  ( select round(sum(bytes)/1024/1024) total_MB,
           round(sum(decode(name,''KGH: NO ACCESS'',bytes/1024/1024,0))) lost_MB
    from dba_hist_sgastat)';
 --
 set_output_time;
 begin
  execute immediate v_sql into total_MB, lost_MB, lost_prc;
  put(null,'Memory leaks',c_error,
    '    SGA     "KGH: NO ACCESS"'||
    '    '||lost_prc||'%'||chr(10)||
    '     -  ,  '||
    '      . (Metalink Note 451960.1)');
 exception when e_table_not_exists then
  put(null,'Memory leaks',c_error,
    '       sys.dba_hist_sgastat');
 end;

 --   
 v_sql :=
   'select total_CPU, parse_CPU, parse_elapsed, parse_elapsed - parse_CPU waiting
   ,trunc(100*parse_elapsed/total_CPU,2) pct_parsing
   ,trunc(parse_elapsed/parse_CPU,2) r1 --  --> 1
   ,trunc(parse_CPU/total_CPU,2) r2     --  --> 0
   from
   (select value/100 total_CPU from v$sysstat where name = ''CPU used by this session'') -- seconds
   ,(select value/100 parse_CPU from v$sysstat where name = ''parse time cpu'')          -- seconds
   ,(select value/100 parse_elapsed from v$sysstat where name = ''parse time elapsed'')  -- seconds';
 --
 set_output_time;
 begin
  execute immediate v_sql into n,n,n,n,pct_parsing,n,n;
  put(null,'Parsing',c_error,
    '       '||pct_parsing
  ||' %    ( SQL-).       CPU, '
  ||'        SQL- -  '
  ||' SQL   bind-   cursor_shring = force. '
  ||' cursor_shring = force     ,   !');
 exception when e_table_not_exists then
  put(null,'Parsing',c_error,
    '       v$sysstat');
 end;
 --
 commit;
end;
--------------------------------------------------------------------------------
procedure run_all is
begin
 check_output_table;
 set_output_time;
 --
 for c in (select procedure_name n from all_procedures
   where owner='UN4PUBLIC' and object_name='UN$SCHEMA_CHECK'
   and procedure_name like 'CHECK\_%' ESCAPE '\') loop
  say('BEGIN UN$SCHEMA_CHECK.'||c.n||';END;');
  execute immediate 'BEGIN UN$SCHEMA_CHECK.'||c.n||';END;';
 end loop;
 --
 commit;
end;
--------------------------------------------------------------------------------
end;
/
